﻿using System;
using System.Data;
using System.Data.OleDb;
using System.Diagnostics;

namespace C1DBSpellChecker
{
    static class Types
    {
        // check whether a given type is numeric.
        public static bool IsNumeric(Type type)
        {
            // handle regular types
            switch (Type.GetTypeCode(type))
            {
                case TypeCode.Decimal:
                case TypeCode.Double:
                case TypeCode.Single:
                case TypeCode.Byte:
                case TypeCode.Int16:
                case TypeCode.Int32:
                case TypeCode.Int64:
                case TypeCode.SByte:
                case TypeCode.UInt16:
                case TypeCode.UInt32:
                case TypeCode.UInt64:
                    return true;
            }

            // handle nullable types
            type = Nullable.GetUnderlyingType(type);
            return type != null
                ? IsNumeric(type)
                : false;
        }

        // translate an OleDbType into a .NET type.
        public static Type GetType(OleDbType oleDbType)
        {
            switch ((int)oleDbType)
            {
                case 0: return typeof(Nullable);
                case 2: return typeof(short);
                case 3: return typeof(int);
                case 4: return typeof(float);
                case 5: return typeof(double);
                case 6: return typeof(decimal);
                case 7: return typeof(DateTime);
                case 8: return typeof(string);
                case 9: return typeof(object);
                case 10: return typeof(Exception);
                case 11: return typeof(bool);
                case 12: return typeof(object);
                case 13: return typeof(object);
                case 14: return typeof(decimal);
                case 16: return typeof(sbyte);
                case 17: return typeof(byte);
                case 18: return typeof(ushort);
                case 19: return typeof(uint);
                case 20: return typeof(long);
                case 21: return typeof(ulong);
                case 64: return typeof(DateTime);
                case 72: return typeof(Guid);
                case 128: return typeof(byte[]);
                case 129: return typeof(string);
                case 130: return typeof(string);
                case 131: return typeof(decimal);
                case 133: return typeof(DateTime);
                case 134: return typeof(TimeSpan);
                case 135: return typeof(DateTime);
                case 138: return typeof(object);
                case 139: return typeof(decimal);
                case 200: return typeof(string);
                case 201: return typeof(string);
                case 202: return typeof(string);
                case 203: return typeof(string);
                case 204: return typeof(byte[]);
                case 205: return typeof(byte[]);
            }
            Debug.WriteLine("** unknown type: " + oleDbType.ToString());
            return typeof(string);
        }

        // translate an Access parameter type (in PARAMETERS clause) into an OleDbType.
        public static OleDbType GetOleDbType(string typeName)
        {
            switch (typeName.ToLower())
            {
                case "Bit": return OleDbType.Boolean;
                case "Byte": return OleDbType.TinyInt;
                case "Short": return OleDbType.SmallInt;
                case "Long": return OleDbType.BigInt;
                case "Currency": return OleDbType.Currency;
                case "IEEESingle": return OleDbType.Single;
                case "IEEEDouble": return OleDbType.Double;
                case "DateTime": return OleDbType.Date;
                case "Text": return OleDbType.VarChar;
                case "Decimal": return OleDbType.Decimal;
            }

            Debug.WriteLine("** unknown type: '{0}'", typeName);
            return OleDbType.VarChar;
        }
    }
}
